////////////////////////////////////////
///////////                //////////////
//////////     语音朗读     /////////////
//////////                 /////////////
////////////////////////////////////////
function AudioPlayer() {
    var playerDom = document.querySelector(".audio");
    var playBar = playerDom.querySelector(".main");
    var playBtn = playerDom.querySelector(".play_btn");
    var durationDom = playerDom.querySelector(".duration");
    var sizeDom = playerDom.querySelector(".size");

    var PLAY_STATE = {
        BUFFING:"buffing",
        PLAYING:"playing",
        PAUSE:"pause"
    }

    var newsId = 0;
    var lastClickTime = 0;

    rm.onReceiveMessage("onNewsAudioPlayStateChanged", function (data) {
        // 返回 {newsId: 0, play: 0}
        if (newsId === data.newsId) {
            setPlayState(data && data.play ? PLAY_STATE.PLAYING : PLAY_STATE.PAUSE);
        }
    })

    playerDom.onclick = function () {
        if(Math.abs(new Date().getTime() - lastClickTime) < 120){
            return;
        }
        lastClickTime = new Date().getTime();
        if(rm.getEnv().appver > 8.64){
            if(isPause()){
                setPlayState(PLAY_STATE.BUFFING);
            }
            rm.sendMessage(
                {
                    name: "switchNewsAudioPlayState",
                    newsId: newsInfo.id
                }
            );
        } else {
            if(rm.getEnv().platform === "Android"){
                rm.showUpgradeTip();
            } else {
                rm.showModal(
                    {
                        content: '使用此功能需升级App到最新版本',
                        cancelText: '以后再说',
                        showCancel: true,
                        confirmText: '升级'
                    },
                    function(res){
                        if (res.confirm) {
                            rm.navigateTo({
                                url: "http://m.ithome.com/ithome/download/",
                            })
                        }
                    }
                )
            }
        }
      
    }

    this.initPlayer = function (newsInfo, config) {
        newsId = newsInfo.id;
        if (newsInfo.newsVoices && newsInfo.newsVoices.length > 0) {
            switchVisible(true);
            var voice = newsInfo.newsVoices[0]
            var durationSeconds = Math.round(voice.duration * 1.0 / 1000.0);
            var minutes = Math.floor(durationSeconds / 60);
            var seconds = durationSeconds % 60;
            durationDom.innerHTML = minutes + "分" + (seconds > 0 ? (seconds + "秒") : '钟');;//pad(minutes, 2) + ":" + pad(seconds, 2);
            var size = voice.length;
            var units = ["B", "KB", "MB", "G"];
            var unitIndex = 0
            while (Math.round(size) > 1024 && unitIndex < units.length - 1) {
                size = size * 1.0 / 1024;
                unitIndex++;
            }
            sizeDom.innerHTML = size.toFixed(1) + units[unitIndex];
            if(rm.getEnv().appver > 8.64){
                rm.sendMessage(
                    {
                        name: "checkNewsAudioPlayState",
                        newsId: newsInfo.id
                    }
                );
            }
        } else {
            switchVisible(false);
        }
    }

    this.release = function () {
        setPlayState(PLAY_STATE.PAUSE)
        switchVisible(false)
    }

    function setPlayState(state) {
        playBtn.setAttribute("data-state", state)
    }
    function isPause() {
        var state = playBtn.getAttribute("data-state");
        return state === PLAY_STATE.PAUSE
    }
    function switchVisible(show) {
        playerDom.classList[show ? 'add' : 'remove']("show")
    }
}

var audioPlayer = new AudioPlayer();
