////////////////////////////////////////
///////////                //////////////
//////////       链接元素   /////////////
//////////                 /////////////
////////////////////////////////////////
var anchorHelper = new AnchorHelper();
function AnchorHelper() {
    this.process = function (container) {
        // 普通链接
        var links = container.querySelectorAll("a");
        for (var i = 0; i < links.length; i++) {
            var link = links[i];
            link.onclick = onClick;

            // 磁力链接处理
            if (
                link.classList.contains("ed2k-link") ||
                link.classList.contains("magnet-link")
            ) {
                var copyDom = document.createElement("span");
                copyDom.classList.add("copyLinkBtn");
                copyDom.innerText = "复制链接";
                copyDom.setAttribute("data-copy", link.getAttribute("href"));
                copyDom.onclick = function (e) {
                    e.preventDefault();
                    e.cancelable = true;
                    e.stopPropagation();
                    var dom = e.currentTarget;
                    rm.vibrate();
                    rm.login(function (res) {
                        if (res.success) {
                            rm.setClipboardData(
                                dom.getAttribute("data-copy"),
                                function (res) {
                                    if (res.success) {
                                        rm.toast({
                                            data: "链接已复制",
                                        });
                                    }
                                }
                            );
                        }
                    })

                };
                link.insertAdjacentElement("afterend", copyDom);
            }

            // 去下载描述文件
            if (link.id === "to-app-iosfiles") {
                link.innerHTML = "客户端内点此直达描述文件下载";
                link.setAttribute(
                    "href",
                    "https://img.ithome.com/app/iosprofiles/index.html?hidemenu=1&fitsystemWindow=1&isrmmp=1&keeptitle=1"
                );
            }

            // 链接中的图片处理
            var imgs = link.querySelectorAll("img");
            var isHighQualityImage = false;
            if ((link.innerText || "").trim().length === 0 && imgs.length === 1) {
                if (!link.classList.contains("ithome_super_player"))
                    link.classList.add("link_rich_content");
                var href = link.getAttribute("href");
                var imgUrl = getRealSrc(imgs[0]);
                if (imgUrl.indexOf(href) === 0) {
                    isHighQualityImage = true;
                    // 是原图模式图片
                    link.setAttribute("href", "javascript:void(0);");
                    imgs[0].setAttribute("data-highquality", href);
                }
            }
            if (!isHighQualityImage) {
                // 非原图模式下链接中的图片都自动显示
                for (var index = 0; index < imgs.length; index++) {
                    var img = imgs[index];
                    var realSrc = getRealSrc(img);
                    if (realSrc && img.getAttribute("src") !== realSrc) {
                        img.setAttribute("src", realSrc);
                    }
                    img.setAttribute("data-noneclick", "yes");
                }
            }
        }

        // 三方平台链接：微博、抖音
        var needCheckPlatforms = [
            {
                scheme: "sinaweibo://",
                host: "userinfo",
            },
        ];
        if (rm.getEnv().platform === "Android") {
            needCheckPlatforms.push({
                scheme: "snssdk1128://",
            });
        }
        needCheckPlatforms.forEach(function (platform) {
            var scheme = platform.scheme;
            var schemeUrl = scheme + (platform.host || "");
            var selector = 'a[data-androidurl*="' + scheme + '"]';
            var targetUrls = container.querySelectorAll(selector);
            if (targetUrls.length > 0) {
                rm.canOpenURL({ url: schemeUrl }, function (res) {
                    if (res.status !== 1) {
                        for (var index = 0; index < targetUrls.length; index++) {
                            var element = targetUrls[index];
                            var url = element.getAttribute("href") || "";
                            if (url.toLowerCase().trim().indexOf("http") === 0) {
                                element.removeAttribute("data-androidurl");
                                element.removeAttribute("data-iosurl");
                            }
                        }
                    }
                });
            }
        });

        //之家辣品文章，根据是否安装最会买，跳转不同的链接
        var zuihuimaiBuy = document.querySelectorAll("a[data-zuihuimaibuy]");
        var zuihuimaidown = document.getElementById("zuihuimaidown");
        if (zuihuimaiBuy.length > 0 || zuihuimaidown) {
            rm.canOpenURL(
                {
                    url: "zuihuimai://openurl",
                    getDetail: false,
                },
                function (res) {
                    if (res.status === 1) {
                        // 最会买购买
                        for (var index = 0; index < zuihuimaiBuy.length; index++) {
                            var element = zuihuimaiBuy[index];
                            var zhmurl = element.getAttribute("data-zuihuimaibuy");
                            element.setAttribute("href", zhmurl);
                            element.removeAttribute("data-iosurl");
                            element.removeAttribute("data-androidurl");
                            element.innerText = "点此下单拿返利";
                        }
                        // 最会买App下载
                        if (zuihuimaidown) {
                            zuihuimaidown.setAttribute("href", "zuihuimai://openurl");
                            zuihuimaidown.removeAttribute("data-iosurl");
                            zuihuimaidown.removeAttribute("data-androidurl");
                            zuihuimaidown.innerText = "打开最会买";
                        }
                    }
                }
            );
        }

        // 之家公众号文章优先跳转到小程序打开，如果检测到未安装微信，则删除链接
        var platAttr =
            rm.getEnv().platform === "iOS" ? "data-iosurl" : "data-androidurl";
        var mpTags = document.querySelectorAll(
            "a[" + platAttr + "^='ithome://openmp?appid=ithome&auto=1']"
        );
        if (mpTags.length > 0) {
            rm.canOpenURL(
                {
                    url: "weixin://dl/",
                },
                function (res) {
                    if (res.status !== 1) {
                        for (var index = 0; index < mpTags.length; index++) {
                            var element = mpTags[index];
                            element.removeAttribute(platAttr);
                        }
                    }
                }
            );
        }

        // 商品卡片
        var cards = document.querySelectorAll(".card.main-site");
        for (var index = 0; index < cards.length; index++) {
            var card = cards[index];
            var mallName = card.querySelector(".mall-data");
            if (mallName) {
                var mallToken = "none";
                switch (mallName.innerText) {
                    case "天猫":
                        mallToken = "tmall";
                        break;
                    case "淘宝":
                        mallToken = "taobao";
                        break;
                    case "京东":
                        mallToken = "jingdong";
                        break;
                    case "拼多多":
                        mallToken = "pinduoduo";
                        break;
                }
                card.classList.add(mallToken);
            }
            var price = card.querySelector(".price");
            if (price) {
                price.innerHTML = price.innerHTML.replace(
                    /(\d+)(\.\d+)?/g,
                    '<span class="price-num">$1$2</span>'
                );
            }
            var getBtn = card.querySelector(".card-btn-deep");
            if (getBtn) {
                getBtn.innerHTML = getBtn.innerHTML.replace(
                    /(\d+)(\.\d+)?/g,
                    '<span class="price-num">$1$2</span>'
                );
            }
        }
    };

    function onClick(e) {
        e.preventDefault();
        var tag = e.currentTarget;
        var url = getPlatformUrl(tag);
        if (!isEmpty(url)) {
            if (url.trim().indexOf("#") === 0) {
                // 锚链
                var to = document.querySelector(url);
                if (to) {
                    var top = to.getBoundingClientRect().top;
                    try {
                        window.scrollTo({ top: top, behavior: "smooth" });
                    } catch (e) {
                        window.scrollTo(0, top);
                    }
                }
            } else if (url.indexOf("javascript:") === 0) {
                // 脚本
                eval(url.substring("javascript:".length));
            } else if (url.indexOf("ed2k") === 0
                || url.indexOf("magnet") === 0) {
                // 磁力/电驴。登录后下载
                rm.login(function (res) {
                    if (res.success) {
                        function copyAndTip() {
                            rm.setClipboardData(
                                url,
                                function (res) {
                                    if (res.success) {
                                        rm.toast({
                                            data: url.indexOf("ed2k") === 0 ? "链接已复制，请使用电驴下载器下载" : "链接已复制，请使用磁力下载器下载"
                                        });
                                    }
                                }
                            );

                        }
                        if (isAndroid) {
                            rm.canOpenURL(
                                {
                                    url: url,
                                    getDetail: true
                                },
                                function (res) {
                                    if (res.status === 1) {
                                        rm.navigateTo({
                                            url: url,
                                        });
                                    } else {
                                        copyAndTip()
                                    }
                                }
                            )
                        } else {
                            copyAndTip()
                        }
                    } else {
                        rm.toast({
                            data: "登录即可下载",
                        });
                    }
                });
            } else {
                // 普通链接
                var needLogin = tag.classList.contains("a-lock");
                if (needLogin && !isLogin()) {
                    toast("登录后查看")
                    rm.login(function (res) {
                        if (res.success) {
                            openLink()
                        }
                    })
                } else {
                    openLink()
                }

            }
        }
        function openLink() {
            var shouldOpenExternal =
                isReviewMode &&
                (url.indexOf("jd.com") !== -1 ||
                    url.indexOf("taobao.com") !== -1 ||
                    url.indexOf("tmall.com") !== -1);
            rm.navigateTo({
                url: url,
                openExternal: shouldOpenExternal,
            });
        }
    }
}
