////////////////////////////////////////
///////////                //////////////
//////////    段落评论工具  /////////////
//////////                 /////////////
////////////////////////////////////////
function QuoteCommentHelper() {
    var that = this;
    this.newsId = 0;
    this.isCommentBubbleEnable = false;
    this.lastQuoteRange = {};
    // 初始化数据
    this.loadData = function (newsInfo) {
        that.newsId = newsInfo.newsid;
        var isCommentEnable = !newsInfo.forbidcomment && !getUserInfo().nocomment;
        if (isCommentEnable && !isShareMode) {
            rm.getSettings({ ignoreCloud: true }, function (res) {
                that.isCommentBubbleEnable =
                    res.success &&
                    (res.settings.enableCommentBubble ||
                        typeof res.settings.enableCommentBubble === "undefined");
                if (that.isCommentBubbleEnable) {
                    loadParagraphComment();
                }
            });
        }
    };
    // 获取选区信息
    this.getSelection = function () {
        var selectHtml = getSelectionHtml();
        var res = {
            html: selectHtml.html,
            text: selectHtml.text,
            newsId: newsId,
        };
        if (window.getSelection && window.getSelection()) {
            var selection = window.getSelection();
            res.startP = getAnchorPmark(selection.anchorNode);
            res.endP = getAnchorPmark(selection.extentNode);
            res.token = uuid();
            that.lastQuoteRange = {
                range: selection.getRangeAt(0).cloneRange(),
                text: res.text,
                token: res.token,
            };
        }
        return res;
    };
    // 定位段落并高亮
    this.highlightNewsParagraph = function (data) {
        var cancelList = document.querySelectorAll("*[data-highlight=quote]");
        for (var index = 0; index < cancelList.length; index++) {
            var element = cancelList[index];
            normalizeNode(element);
        }
        var id = data.id;
        var text = data.text;
        var dom = document.querySelector("*[data-vmark*='" + id + "']");
        if (id.toLowerCase() === "head") {
            dom = document.querySelector("h1");
        }
        var to = 0;
        if (dom) {
            var rect = dom.getBoundingClientRect();
            to =
                rect.top -
                document.body.getBoundingClientRect().top -
                window.innerHeight / 3;
            if (!data.onlyGetPosition) {
                window.scrollTo(0, to);
            }
            dom.classList.add("highlight-quote-bg");
            setTimeout(function () {
                dom.classList.remove("highlight-quote-bg");
            }, 520);
            try {
                setTimeout(function () {
                    highlightKeyWords(dom, [
                        {
                            key: text,
                            tip: "",
                            singleindex: -1,
                        },
                    ]);
                }, 650);
            } catch (error) { }
        }
        return {
            success: !!dom,
            message: !!dom ? "已找到段落" : "段落已失去关联",
            top: to,
        };
    };

    // 评论发布后高亮
    this.quoteCommentSubmit = function (data) {
        var info = typeof data === "string" ? JSON.parse(data) : data;
        var pmark = info.startP;

        var selector = "*[data-vmark='" + pmark + "']";
        if (pmark === "head") {
            selector = ".title";
        }

        var p = document.querySelector(selector);
        if (p && !isEmpty(p.innerText, true)) {
            // 更新评论数量
            var mark = p.querySelector(".comment-quote-mark");
            if (!mark) {
                mark = document.createElement("span");
                mark.className = "comment-quote-mark start-one";
                mark.innerHTML = "<span>" + 1 + "</span>";
                mark.setAttribute("role", "option");
                mark.setAttribute("aria-label", "段落评论," + 1 + "条");
                p.insertAdjacentElement("beforeend", mark);
            } else {
                var currentCount = parseInt(mark.innerText);
                var newCount = currentCount + 1;
                mark.innerHTML = "<span>" + newCount + "</span>";
                mark.setAttribute("aria-label", "段落评论," + newCount + "条");
                if (newCount > 9) {
                    mark.classList.add("red");
                }
                if ((newCount + "").indexOf("1") === 0) {
                    mark.classList.add("start-one");
                } else {
                    mark.classList.remove("start-one");
                }
            }
            mark.onclick = function () {
                rm.vibrate();
                rm.navigateTo({
                    url:
                        "ithome://paragraphcomment?newsid=" +
                        that.newsId +
                        "&paragraphid=" +
                        pmark,
                });
            };
            // 引用画横线
            if (
                info.token === that.lastQuoteRange.token &&
                info.startP === info.endP &&
                !p.querySelector("dir")
            ) {
                var newdom = document.createElement("span");
                newdom.innerText = that.lastQuoteRange.text;
                newdom.className = "comment-quote-element";
                that.lastQuoteRange.range.deleteContents();
                that.lastQuoteRange.range.insertNode(newdom);
            }
        }
    };
    rm.onReceiveMessage("quoteCommentSubmit", function (data) {
        if (that.isCommentBubbleEnable) {
            that.quoteCommentSubmit(data);
        }
    });
    rm.onReceiveMessage("highLightNewsParagraph", function (data, callback) {
        callback(that.highlightNewsParagraph(data));
    });
    rm.onReceiveMessage("getSelection", function (data, callback) {
        callback(that.getSelection());
    });

    function loadParagraphComment() {
        // 加载段落评论
        rm.request(
            {
                url:
                    "https://cmt.ithome.com/api/comment/getparagraphcommentcounts?sn=" +
                    rm.encryptSync({ text: that.newsId + "" }).data,
            },
            function (res) {
                if (res.statusCode === 200 && res.data.success) {
                    var ps = res.data.content;
                    ps &&
                        ps.forEach(function (item) {
                            var selector = "*[data-vmark='" + item.paragraphId + "']";
                            if (item.paragraphId === "head") {
                                selector = ".title";
                            }
                            var p = document.querySelector(selector);
                            if (p && !isEmpty(p.innerText, true)) {
                                // 更新评论数量
                                var mark = p.querySelector(".comment-quote-mark");
                                if (!mark) {
                                    mark = document.createElement("span");
                                    var pText = (p.innerText || "").trim();
                                    var endWithMark =
                                        ["。", "！", "？", "：", "》", "；", "）", "】"].indexOf(
                                            pText[pText.length - 1]
                                        ) !== -1;
                                    mark.className =
                                        "comment-quote-mark" + (!endWithMark ? " large" : "");
                                    p.insertAdjacentElement("beforeend", mark);
                                }
                                if ((item.count + "").indexOf("1") === 0) {
                                    mark.classList.add("start-one");
                                } else {
                                    mark.classList.remove("start-one");
                                }
                                if (item.count > 9) {
                                    mark.classList.add("red");
                                }
                                mark.innerHTML = "<span>" + item.count + "</span>";
                                mark.setAttribute("role", "option");
                                mark.setAttribute(
                                    "aria-label",
                                    "段落评论," + item.count + "条"
                                );
                                mark.onclick = function () {
                                    rm.vibrate();
                                    rm.navigateTo({
                                        url:
                                            "ithome://paragraphcomment?newsid=" +
                                            that.newsId +
                                            "&paragraphid=" +
                                            item.paragraphId,
                                    });
                                };
                            }
                        });
                }
            }
        );
    }

    function getAnchorPmark(target) {
        var pMark = "";
        var anchorNode = target;
        var searchTime = 1;
        while (
            anchorNode &&
            anchorNode.tagName !== "P" &&
            anchorNode.tagName !== "H1" &&
            anchorNode.tagName !== "H2" &&
            anchorNode.tagName !== "DIV" &&
            (anchorNode.parentNode || anchorNode.parentElement) &&
            searchTime < 20
        ) {
            searchTime++;
            anchorNode = anchorNode.parentNode || anchorNode.parentElement;
        }
        if (anchorNode && anchorNode.hasAttribute("data-vmark")) {
            pMark = anchorNode.getAttribute("data-vmark");
        } else if (anchorNode && anchorNode.tagName === "H1") {
            pMark = "head";
        }
        return pMark;
    }
    
    function getSelectionHtml() {
        var html = "";
        if (typeof window.getSelection != "undefined") {
            var sel = window.getSelection();
            if (sel.rangeCount) {
                var container = document.createElement("div");
                for (var i = 0, len = sel.rangeCount; i < len; ++i) {
                    container.appendChild(sel.getRangeAt(i).cloneContents());
                }
                html = container.innerHTML;
            }
            if (sel.anchorNode && (!html || html.length === 0)) {
                html = sel.anchorNode.textContent || sel.anchorNode.nodeValue;
            }
        } else if (typeof document.selection != "undefined") {
            if (document.selection.type == "Text") {
                html = document.selection.createRange().htmlText;
            }
        }
        var container = document.createElement("div");
        container.innerHTML = html;
        var text = container.innerText;
        return {
            html: html,
            text: text,
        };
    }
}
var quoteCommentHelper = new QuoteCommentHelper();